from tempfile import TemporaryFile

import StringIO
from django.conf import settings
from django.core.files.storage import Storage
from django.core.files.base import File
from backblaze_b2 import BackBlazeB2


class B2Storage(Storage):
    def __init__(self, account_id=None, app_key=None, bucket_name=None):
        self.account_id = settings.BACKBLAZEB2_ACCOUNT_ID  # if account_id == None
        self.app_key = settings.BACKBLAZEB2_APP_KEY  # if app_key == None
        self.bucket_name = settings.BACKBLAZEB2_BUCKET_NAME  # if bucket_name == None
        self.b2 = BackBlazeB2(app_key=self.app_key, account_id=self.account_id, bucket_name=self.bucket_name)

    def save(self, name, content, max_length=None):
        """
        Save and retrieve the filename.
        If the file exists it will make another version of that file.
        """

        resp = self.b2.upload_file(name, content)
        if 'fileName' in resp:
            return resp['fileName']

        else:
            # Raise exception
            pass

    def exists(self, name):
        '''
        BackBlaze B2 does not have a method to retrieve a filename info.
        To get the info you need to make a download request, it will request the whole body.
        imagine a file of 1 GB to only get the file info.
        you can also list all files in that directory in chunks of 1000 imagine a directory of 10000.
        For now it will only request return False.
        '''

        return False

    def _temporary_storage(self, contents):
        '''
        Use this to return file objects
        '''

        conent_file = TemporaryFile(contents, 'r+')
        return conent_file

    def open(self, name, mode='rb'):
        resp = self.b2.download_file(name)

        output = StringIO.StringIO()
        output.write(resp)
        output.seek(0)
        return File(output, name)


        #
        # def get_available_name(self, name, max_length=None):
        #     pass
        #
        # def delete(self, name):
        #     pass
        #
        # def exists(self, name):
        #     pass
        #
        # def listdir(self, path):
        #     pass
        #
        # def size(self, name):
        #     pass
        #
        # def url(self, name):
        #     pass



import base64
from contextlib import closing

import requests
import hashlib


class BackBlazeB2(object):
    def __init__(self, app_key=None, account_id=None, bucket_name=None):
        self.bucket_id = None
        self.account_id = account_id
        self.app_key = app_key
        self.bucket_name = bucket_name
        self.authorize()
        self.get_bucket_id_by_name()

    def authorize(self):
        headers = {'Authorization': 'Basic: %s' % (base64.b64encode('%s:%s' % (self.account_id, self.app_key)))}
        response = requests.get('https://api.backblaze.com/b2api/v1/b2_authorize_account', headers=headers)
        if response.status_code == 200:
            resp = response.json()
            self.base_url = resp['apiUrl']
            self.download_url = resp['downloadUrl']
            self.authorization_token = resp['authorizationToken']

            return True

        else:
            return False

    def get_upload_url(self):
        url = self._build_url('/b2api/v1/b2_get_upload_url')
        headers = {'Authorization': self.authorization_token}
        params = {'bucketId': self.bucket_id}
        return requests.get(url, headers=headers, params=params).json()

    def _build_url(self, endpoint=None, authorization=True):
        return "%s%s" % (self.base_url, endpoint)

    def upload_file(self, name, content):
        response = self.get_upload_url()
        if 'uploadUrl' not in response:
            return False

        url = response['uploadUrl']
        sha1_of_file_data = hashlib.sha1(content.read()).hexdigest()
        content.seek(0)

        headers = {
            'Authorization': response['authorizationToken'],
            'X-Bz-File-Name': name,
            'Content-Type': "b2/x-auto",
            'X-Bz-Content-Sha1': sha1_of_file_data,
            'X-Bz-Info-src_last_modified_millis': '',
        }

        download_response = requests.post(url, headers=headers, data=content.read())
        if download_response.status_code != 200:
            # raise exception here.
            pass
        else:
            pass

        return download_response.json()

    def get_file_info(self, name):
        headers = {'Authorization': self.authorization_token}
        return requests.get("%s/file/%s/%s" % (self.download_url, self.bucket_name, name), headers=headers)

    def download_file(self, name):
        headers = {'Authorization': self.authorization_token}
        return requests.get("%s/file/%s/%s" % (self.download_url, self.bucket_name, name), headers=headers).content

    def get_bucket_id_by_name(self):
        """
        BackBlaze B2 should  make an endpoint to retrieve buckets by its name.
        """
        headers = {'Authorization': self.authorization_token}
        params = {'accountId': self.account_id}
        resp = requests.get(self._build_url("/b2api/v1/b2_list_buckets"), headers=headers, params=params).json()
        if 'buckets' in resp:
            buckets = resp['buckets']
            for bucket in buckets:
                if bucket['bucketName'] == self.bucket_name:
                    self.bucket_id = bucket['bucketId']
                    return True

        else:
            return False



from setuptools import setup, find_packages

from b2_storage import __version__

version_str = ".".join(str(n) for n in __version__)

setup(
        name="django-backblazeb2-storage",
        version=version_str,
        license="BSD",
        description="Django BackBlaze B2 Storage",
        author="Royendgel Silberie",
        author_email="rsilberie@techprocur.com",
        url="https://github.com/royendgel/django-blackblazeb2-storage",
        packages=find_packages(),
        classifiers=[
            "Development Status :: 4 - Beta",
            "Environment :: Web Environment",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.3",
            "Programming Language :: Python :: 3.4",
            "Framework :: Django",
        ],
)
